@@ -13,7 +13,7 @@
   performance for heterogeneity.
 */
 
-/* $Id: arrayobject.c,v 1.39 2001/12/10 20:28:46 dubois Exp $ */
+/* $Id: arrayobject.c,v 1.58 2002/08/28 17:37:01 dubois Exp $ */
 
 #include "Python.h"
 /*Silly trick to make dll library work right under Win32*/
@@ -66,7 +66,7 @@ int do_sliced_copy(char *dest, int *dest_strides, int *dest_dimensions,
 	
 	if (src_nd == 0 && dest_nd == 0) {
 		for(j=0; j<copies; j++) {
-			memcpy(dest, src, elsize);
+			memmove(dest, src, elsize);
 			dest += elsize;
 		}
 		return 0;
@@ -92,7 +92,7 @@ int do_sliced_copy(char *dest, int *dest_strides, int *dest_dimensions,
 		}
 		for(i=0; i<*dest_dimensions; i++, src += *src_strides) {
 			for(j=0; j<copies; j++) {
-				memcpy(dest, src, elsize);
+				memmove(dest, src, elsize);
 				dest += *dest_strides;
 			}
 		}
@@ -296,7 +296,7 @@ int PyArray_CopyObject(PyArrayObject *dest, PyObject *src_object) {
 	n_old = PyString_Size(src_object); 
 	if (n_new > n_old) {
 	    new_string = (char *)malloc(n_new*sizeof(char));
-	    memcpy(new_string, 
+	    memmove(new_string, 
 		   PyString_AS_STRING((PyStringObject *)src_object),
 		   n_old);
 	    memset(new_string+n_old, ' ', n_new-n_old);
@@ -342,7 +342,7 @@ PyObject *PyArray_FromDimsAndDataAndDescr(int nd, int *d,
 	    PyErr_SetString(PyExc_MemoryError, "can't allocate memory for array");
 	    goto fail;
 	}
-	memcpy(dimensions, d, sizeof(int)*nd);
+	memmove(dimensions, d, sizeof(int)*nd);
     }
 	
     sd = descr->elsize;
@@ -521,7 +521,7 @@ static PyObject * PyArray_Resize(PyArrayObject *self, PyObject *shape) {
 	    dptr = self->data + oldsize*elsize;
 	    n = newsize - oldsize;
 	    for(k=0; k<n; k++) {
-		memcpy(dptr, self->descr->zero, elsize);
+		memmove(dptr, self->descr->zero, elsize);
 		dptr += elsize;
 	    }
 	}
@@ -546,8 +546,8 @@ static PyObject * PyArray_Resize(PyArrayObject *self, PyObject *shape) {
 	self->strides = strptr;
     }
 
-    memcpy(self->dimensions, new_dimensions, new_nd*sizeof(int));
-    memcpy(self->strides, new_strides, new_nd*sizeof(int));
+    memmove(self->dimensions, new_dimensions, new_nd*sizeof(int));
+    memmove(self->strides, new_strides, new_nd*sizeof(int));
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -586,15 +586,15 @@ static PyObject *array_item(PyArrayObject *self, int i) {
 	
     if(self->nd > 1) {
 	PyArrayObject *r;
-	if (( r= PyObject_NEW(PyArrayObject, &PyArray_Type)) == NULL) return NULL;
-
-	r->nd = self->nd-1;
-	r->dimensions = self->dimensions+1;
-	r->strides = self->strides+1;
-	r->descr = self->descr;
-	r->data = item;
+        r = (PyArrayObject *)PyArray_FromDimsAndDataAndDescr(self->nd-1,
+                                                             self->dimensions+1,
+                                                             self->descr,
+                                                             item);
+        if (r == NULL) return NULL;
+        memmove(r->strides, self->strides+1, sizeof(int)*(r->nd));
 	r->base = (PyObject *)self;
-	r->flags = self->flags & (CONTIGUOUS | SAVESPACE);
+	r->flags = (self->flags & (CONTIGUOUS | SAVESPACE));
+        r->flags |= OWN_DIMENSIONS | OWN_STRIDES;
 	Py_INCREF(self);
 	return (PyObject*)r;
     } else {
@@ -622,8 +622,13 @@ extern PyObject *PyArray_Return(PyArrayObject *mp) {
 	if (mp != NULL) {Py_DECREF(mp);}
 	return NULL;
     }
-    if (mp->nd == 0 && (mp->descr->type_num == PyArray_LONG || mp->descr->type_num == PyArray_DOUBLE || mp->descr->type_num == PyArray_CDOUBLE || mp->descr->type_num == PyArray_OBJECT )) {
-	op = mp->descr->getitem(mp->data);
+    if (mp->nd == 0) {
+	if (mp->descr->type_num == PyArray_LONG || mp->descr->type_num == PyArray_DOUBLE || mp->descr->type_num == PyArray_CDOUBLE || mp->descr->type_num == PyArray_OBJECT ) {
+	    op = mp->descr->getitem(mp->data);
+	}
+        else {
+	    op = PyArray_Copy(mp);
+	}
 	Py_DECREF(mp);
 	return op;
     } else {
@@ -663,7 +668,7 @@ static PyObject * array_slice(PyArrayObject *self, int ilow, int ihigh) {
 	
     if (!ISCONTIGUOUS(self)) r->flags &= ~CONTIGUOUS;
     if (self->flags & SAVESPACE) r->flags |= SAVESPACE;
-    memcpy(r->strides, self->strides, sizeof(int)*self->nd);
+    memmove(r->strides, self->strides, sizeof(int)*self->nd);
     r->base = (PyObject *)self;
     Py_INCREF(self);
 	
@@ -688,11 +693,15 @@ static int array_ass_item(PyArrayObject *self, int i, PyObject *v) {
 	if((tmp = (PyArrayObject *)array_item(self, i)) == NULL) return -1;
 	ret = PyArray_CopyObject(tmp, v);
 	Py_DECREF(tmp);
-	return ret;
+	return ret;   
     }
 	
     if ((item = index2ptr(self, i)) == NULL) return -1;
-	
+    /* dvhart - test
+    printf("arrayobject1: ");
+    PyObject_Print(PyObject_Type(v),stdout,0);
+    printf("\n");
+    */
     if(self->descr->type_num != PyArray_OBJECT && PyString_Check(v) && PyObject_Length(v) == 1) {
 	char *s;
 	if ((s=PyString_AsString(v)) == NULL) return -1;
@@ -703,7 +712,11 @@ static int array_ass_item(PyArrayObject *self, int i, PyObject *v) {
 	if(s) c=PyInt_FromLong((long)*s);
 	if(c) v=c;
     }
-	
+    /* dvhart - test
+    printf("arrayobject2: ");
+    PyObject_Print(PyObject_Type(v),stdout,0);
+    printf("\n");
+    */
     self->descr->setitem(v, item);
 	
     if(c) {Py_DECREF(c);}
@@ -751,8 +764,11 @@ static int slice_GetIndices(PySliceObject *r, int length,
 	*stop = PyInt_AsLong(r->stop);
 	if (*stop < 0) *stop += length;
     }
-
-    if (*start > (length-1)) *start = length;
+    if (*step < 0) {
+        if (*start > (length-1)) *start = length-1;
+    } else {
+        if (*start > length) *start = length;
+    }
     if (*start < 0) *start = 0;
     if (*stop < -1) *stop = -1;
     else if (*stop > length) *stop = length;
@@ -820,6 +836,7 @@ static int parse_subindex(PyObject *op, int *step_size, int *n_steps, int max) {
 	}
 	return tmp;
     } 
+
     PyErr_SetString(PyExc_IndexError, 
 		    "each subindex must be either a slice, an integer, Ellipsis, or NewAxis");
     return -1;
@@ -832,14 +849,16 @@ static int parse_index(PyArrayObject *self, PyObject *op,
     int step_size, n_steps;
     PyObject *op1=NULL;
     int is_slice;
-	
+
+
     if (PySlice_Check(op) || op == Py_Ellipsis) {
 	n = 1;
 	op1 = op;
 	Py_INCREF(op);  
 	/* this relies on the fact that n==1 for loop below */
 	is_slice = 1;
-    } else {
+    }
+    else {
 	if (!PySequence_Check(op)) {
 	    PyErr_SetString(PyExc_IndexError, 
 			    "index must be either an int or a sequence");
@@ -851,6 +870,10 @@ static int parse_index(PyArrayObject *self, PyObject *op,
 	
     nd_old = nd_new = 0;
 	
+    /* IF we have at least two args with the last being a FLAG|FUNCTION
+        do me...
+    */
+
     offset = 0;
     for(i=0; i<n; i++) {
 	if (!is_slice) {
@@ -859,7 +882,7 @@ static int parse_index(PyArrayObject *self, PyObject *op,
 		return -1;
 	    }
 	}
-		
+	
 	start = parse_subindex(op1, &step_size, &n_steps, 
 			       nd_old < self->nd ? self->dimensions[nd_old] : 0);
 	Py_DECREF(op1);
@@ -934,7 +957,7 @@ static PyObject *array_subscript(PyArrayObject *self, PyObject *op) {
 								  self->data+offset)) == NULL) {
 	return NULL;
     }
-    memcpy(other->strides, strides, sizeof(int)*other->nd);
+    memmove(other->strides, strides, sizeof(int)*other->nd);
     other->base = (PyObject *)self;
     Py_INCREF(self);
 	
@@ -958,7 +981,7 @@ static PyObject *array_subscript(PyArrayObject *self, PyObject *op) {
 
 static PyObject *array_subscript_nice(PyArrayObject *self, PyObject *op) {
     PyObject *ret;
-        
+       
     if ((ret = array_subscript(self, op)) == NULL) return NULL;
     if (PyArray_Check(ret)) return PyArray_Return((PyArrayObject *)ret);
     else return ret;
@@ -1049,18 +1072,13 @@ static int array_getreadbuf(PyArrayObject *self, int segment, void **ptrptr) {
         while ((product != num_segments) && (i < self->nd)) {
             product *= self->dimensions[i++];
         }
-        
         /* now get the right segment pointer (i is indexing break in contiguity) */
         *ptrptr = self->data + get_segment_pointer(self,segment,i-1);
     } else {
         *ptrptr = self->data;
     }
     /* return the size in bytes of the segment (i is indexing break in contiguity) */
-    product=1;
-    while (i < self->nd) {
-        product *= self->strides[i++];
-    }
-    return product;
+    return NBYTES(self);
 }
 
 
@@ -1104,6 +1122,8 @@ typedef struct {
         *not_equal,
         *greater,
         *greater_equal;
+    PyUFuncObject *floor_divide, /* Added by Bruce Sherwood */
+        *true_divide;            /* for floor and true divide */
 } NumericOps;
 
 static NumericOps n_ops;
@@ -1131,6 +1151,8 @@ int PyArray_SetNumericOps(PyObject *dict) {
     GET(not_equal);
     GET(greater);
     GET(greater_equal);
+    GET(floor_divide);  /* Added by Bruce Sherwood */
+    GET(true_divide);   /* for floor and true divide */
     return 0;
 }
 
@@ -1175,6 +1197,7 @@ static PyObject *PyUFunc_BinaryFunction(PyUFuncObject *s, PyArrayObject *mp1, Py
     arglist = Py_BuildValue("(OO)", mp1, mp2);
     mps[0] = mps[1] = mps[2] = NULL;
     if (PyUFunc_GenericFunction(s, arglist, mps) == -1) {
+	Py_DECREF(arglist);
 	Py_XDECREF(mps[0]); Py_XDECREF(mps[1]); Py_XDECREF(mps[2]);
 	return NULL;
     }
@@ -1184,7 +1207,6 @@ static PyObject *PyUFunc_BinaryFunction(PyUFuncObject *s, PyArrayObject *mp1, Py
     return PyArray_Return(mps[2]);
 }
 
-/*Added by Antoni A. Allen Feb 2001*/
 /*This method adds the augmented assignment*/
 /*functionality that was made available in Python 2.0*/
 static PyObject *PyUFunc_InplaceBinaryFunction(PyUFuncObject *s, PyArrayObject *mp1, PyObject *mp2) {
@@ -1195,6 +1217,7 @@ static PyObject *PyUFunc_InplaceBinaryFunction(PyUFuncObject *s, PyArrayObject *
 	
     mps[0] = mps[1] = mps[2] = NULL;
     if (PyUFunc_GenericFunction(s, arglist, mps) == -1) {
+	Py_DECREF(arglist);
 	Py_XDECREF(mps[0]); Py_XDECREF(mps[1]); Py_XDECREF(mps[2]);
 	return NULL;
     }
@@ -1203,7 +1226,6 @@ static PyObject *PyUFunc_InplaceBinaryFunction(PyUFuncObject *s, PyArrayObject *
     Py_DECREF(arglist);
     return PyArray_Return(mps[2]);
 }
-/*End method added by Antoni A. Allen*/
 
 static PyObject *PyUFunc_UnaryFunction(PyUFuncObject *s, PyArrayObject *mp1) {
     PyObject *arglist;
@@ -1213,6 +1235,7 @@ static PyObject *PyUFunc_UnaryFunction(PyUFuncObject *s, PyArrayObject *mp1) {
 	
     mps[0] = mps[1] = NULL;
     if (PyUFunc_GenericFunction(s, arglist, mps) == -1) {
+	Py_DECREF(arglist);
 	Py_XDECREF(mps[0]); Py_XDECREF(mps[1]);
 	return NULL;
     }
@@ -1266,7 +1289,6 @@ static PyObject *array_bitwise_xor(PyArrayObject *m1, PyObject *m2) {
 }
 
 
-/*Added by Antoni A. Allen Feb 2001*/
 /*These methods add the augmented assignment*/
 /*functionality that was made available in Python 2.0*/
 static PyObject *array_inplace_add(PyArrayObject *m1, PyObject *m2) {
@@ -1302,9 +1324,25 @@ static PyObject *array_inplace_bitwise_or(PyArrayObject *m1, PyObject *m2) {
 static PyObject *array_inplace_bitwise_xor(PyArrayObject *m1, PyObject *m2) {
     return PyUFunc_InplaceBinaryFunction(n_ops.bitwise_xor, m1, m2);
 }
-/*End of methods added by Antoni A. Allen*/
 
+/*Added by Bruce Sherwood Dec 2001*/
+/*These methods add the floor and true division*/
+/*functionality that was made available in Python 2.2*/
+static PyObject *array_floor_divide(PyArrayObject *m1, PyObject *m2) {
+    return PyUFunc_BinaryFunction(n_ops.floor_divide, m1, m2);
+}
+static PyObject *array_true_divide(PyArrayObject *m1, PyObject *m2) {
+    return PyUFunc_BinaryFunction(n_ops.true_divide, m1, m2);
+}
+static PyObject *array_inplace_floor_divide(PyArrayObject *m1, PyObject *m2) {
+    return PyUFunc_InplaceBinaryFunction(n_ops.floor_divide, m1, m2);
+}
+static PyObject *array_inplace_true_divide(PyArrayObject *m1, PyObject *m2) {
+    return PyUFunc_InplaceBinaryFunction(n_ops.true_divide, m1, m2);
+}
+/*End of methods added by Bruce Sherwood*/
 
+/* Array evaluates as "true" if any of the elements are non-zero */
 static int array_nonzero(PyArrayObject *mp) {
     char *zero;
     PyArrayObject *self;
@@ -1322,7 +1360,7 @@ static int array_nonzero(PyArrayObject *mp) {
     }
 	
     Py_DECREF(self);
-	
+
     return i!=s;
 }
 
@@ -1479,7 +1517,6 @@ static PyNumberMethods array_as_number = {
     (unaryfunc)array_oct,	            /*nb_oct*/
     (unaryfunc)array_hex,	            /*nb_hex*/
 
-    /*Added by Antoni A. Allen Feb 2001*/
     /*This code adds augmented assignment functionality*/
     /*that was made available in Python 2.0*/
     (binaryfunc)array_inplace_add,          /*inplace_add*/
@@ -1493,7 +1530,15 @@ static PyNumberMethods array_as_number = {
     (binaryfunc)array_inplace_bitwise_and,  /*inplace_and*/
     (binaryfunc)array_inplace_bitwise_xor,  /*inplace_xor*/
     (binaryfunc)array_inplace_bitwise_or,   /*inplace_or*/
-    /*End of code added by Antoni A. Allen*/
+
+        /* Added in release 2.2 */
+	/* The following require the Py_TPFLAGS_HAVE_CLASS flag */
+#if PY_VERSION_HEX >= 0x02200000
+	(binaryfunc)array_floor_divide,          /*nb_floor_divide*/
+	(binaryfunc)array_true_divide,           /*nb_true_divide*/
+	(binaryfunc)array_inplace_floor_divide,  /*nb_inplace_floor_divide*/
+	(binaryfunc)array_inplace_true_divide,   /*nb_inplace_true_divide*/
+#endif
 };
 
 static PySequenceMethods array_as_sequence = {
@@ -1509,7 +1554,7 @@ static PySequenceMethods array_as_sequence = {
 /* -------------------------------------------------------------- */
 
 /*This ugly function quickly prints out an array.  It's likely to disappear */
-/*in the near future. */
+/*in the near future. (or maybe not...) */
 static int dump_data(char **string, int *n, int *max_n, 
                      char *data, int nd, int *dimensions, 
                      int *strides, PyArray_Descr *descr) {
@@ -1528,7 +1573,7 @@ static int dump_data(char **string, int *n, int *max_n,
 	N = PyString_Size(sp)*sizeof(char);
 	*n += N;
 	CHECK_MEMORY
-	    memcpy(*string+(*n-N), ostring, N);
+	    memmove(*string+(*n-N), ostring, N);
 	Py_DECREF(sp);
 	Py_DECREF(op);
 	return 0;
@@ -1538,7 +1583,7 @@ static int dump_data(char **string, int *n, int *max_n,
 	    *n += N;
 	    CHECK_MEMORY
 		(*string)[*n-N] = '"';
-	    memcpy(*string+(*n-N+1), data, N-2);
+	    memmove(*string+(*n-N+1), data, N-2);
 	    (*string)[*n-1] = '"';
 	    return 0;
 	} else {
@@ -1766,7 +1811,10 @@ PyObject *array_cast(PyArrayObject *self, PyObject *args) {
     if (!PyArg_ParseTuple(args, "O", &op)) return NULL;
 	
     if (PyString_Check(op) && (PyString_Size(op) == 1)) {
-	return PyArray_Cast(self, PyString_AS_STRING((PyStringObject *)op)[0]);
+        typecode = PyString_AS_STRING((PyStringObject *)op)[0];
+        if (PyArray_ValidType(typecode)) return PyArray_Cast(self, typecode);
+        PyErr_SetString(PyExc_ValueError, "Invalid type for array");
+        return NULL;
     }
 	
     if (PyType_Check(op)) {
@@ -2005,7 +2053,7 @@ static PyObject *array_getattr(PyArrayObject *self, char *name) {
 							   self->descr->type_num-2,
 							   self->data);
 	    if (ret == NULL) return NULL;
-	    memcpy(ret->strides, self->strides, sizeof(int)*ret->nd);
+	    memmove(ret->strides, self->strides, sizeof(int)*ret->nd);
 	    ret->flags &= ~CONTIGUOUS;
 	    Py_INCREF(self);
 	    ret->base = (PyObject *)self;
@@ -2029,7 +2077,7 @@ static PyObject *array_getattr(PyArrayObject *self, char *name) {
 							   self->descr->type_num-2,
 							   self->data+self->descr->elsize/2);
 	    if (ret == NULL) return NULL;
-	    memcpy(ret->strides, self->strides, sizeof(int)*ret->nd);
+	    memmove(ret->strides, self->strides, sizeof(int)*ret->nd);
 	    ret->flags &= ~CONTIGUOUS;
 	    Py_INCREF(self);
 	    ret->base = (PyObject *)self;
@@ -2087,7 +2135,7 @@ static int array_setattr(PyArrayObject *self, char *name, PyObject *op) {
 							  self->descr->type_num-2,
 							  self->data);
 	    if (ap == NULL) return -1;
-	    memcpy(ap->strides, self->strides, sizeof(int)*ap->nd);
+	    memmove(ap->strides, self->strides, sizeof(int)*ap->nd);
 	    ap->flags &= ~CONTIGUOUS;
 	    ret = PyArray_CopyObject(ap, op);
 	    Py_DECREF(ap);
@@ -2105,7 +2153,7 @@ static int array_setattr(PyArrayObject *self, char *name, PyObject *op) {
 							  self->descr->type_num-2,
 							  self->data+self->descr->elsize/2);
 	    if (ap == NULL) return -1;
-	    memcpy(ap->strides, self->strides, sizeof(int)*ap->nd);
+	    memmove(ap->strides, self->strides, sizeof(int)*ap->nd);
 	    ap->flags &= ~CONTIGUOUS;
 	    ret = PyArray_CopyObject(ap, op);
 	    Py_DECREF(ap);
@@ -2122,7 +2170,7 @@ static int array_setattr(PyArrayObject *self, char *name, PyObject *op) {
 }
 
 static char Arraytype__doc__[] = 
-"A array object represents a multidimensional, homogeneous array of basic values.  It has the folowing data members, m.shape (the size of each dimension in the array), m.itemsize (the size (in bytes) of each element of the array), and m.typecode (a character representing the type of the matrices elements).  Matrices are sequence, mapping and numeric objects.  Sequence indexing is similar to lists, with single indices returning a reference that points to the old matrices data, and slices returning by copy.  A array is also allowed to be indexed by a sequence of sequences or ints.  Each member of the sequence indexes the corresponding dimension of the array.  If it is a sequence, it returns the elments along that dimension listed in the sequence, if a single number, it returns just that element (reducing the dimensionality of the returned array by 1.  Numeric operations operate on matrices in an element-wise fashion.";
+"A array object represents a multidimensional, homogeneous array of basic values.  It has the folowing data members, m.shape (the size of each dimension in the array), m.itemsize (the size (in bytes) of each element of the array), and m.typecode (a character representing the type of the matrices elements).  Matrices are sequence, mapping and numeric objects.  Sequence indexing is similar to lists, with single indices returning a reference that points to the old matrices data, and slices returning by copy.  A array is also allowed to be indexed by a sequence of items.  Each member of the sequence indexes the corresponding dimension of the array. Numeric operations operate on matrices in an element-wise fashion.";
 
 
 PyTypeObject PyArray_Type = { 
@@ -2149,7 +2197,10 @@ PyTypeObject PyArray_Type = {
     (getattrofunc)0L,                     /*tp_getattro*/
     (setattrofunc)0L,                     /*tp_setattro*/
     (PyBufferProcs *)&array_as_buffer,    /*tp_as_buffer*/
-    (Py_TPFLAGS_HAVE_INPLACEOPS |
+    (Py_TPFLAGS_HAVE_INPLACEOPS | 
+#if PY_VERSION_HEX >= 0x02200000
+         Py_TPFLAGS_HAVE_CLASS |
+#endif
 #if PY_VERSION_HEX >= 0x02010000
 	 Py_TPFLAGS_HAVE_RICHCOMPARE |
 #endif
@@ -2221,6 +2272,7 @@ static int discover_dimensions(PyObject *s, int nd, int *d) {
 static int array_objecttype(PyObject *op, int minimum_type, int savespaceflag) {
     int l;
     PyObject *ip;
+    int result;
 	
     if (minimum_type == -1) return -1;
 	
@@ -2228,14 +2280,18 @@ static int array_objecttype(PyObject *op, int minimum_type, int savespaceflag) {
 	
     if (PyInstance_Check(op)) {
 	if (PyObject_HasAttrString(op, "__array__")) {
-	    PyObject *ap, *arglist;
-			
-	    arglist = Py_BuildValue("()");
-	    ap = PyObject_GetAttrString(op, "__array__");
-	    ip = PyEval_CallObject(ap, arglist);
-	    Py_DECREF(arglist);
-			
-	    return max(minimum_type, (int)((PyArrayObject *)ip)->descr->type_num);
+	    ip = PyObject_CallMethod(op, "__array__", NULL);
+            if(!ip) {
+                /* the original code seems to make no provision for the __array__ */
+                /* call to fail. I do this is a fallback, and the */
+                /* actual call to __array__ does get checked elsewhere. */
+                /* maybe ok?  Probably the Python error flag is on at this point*/
+                /* -- Dubois 3/2002 */
+		return (int)PyArray_OBJECT;
+            }
+	    result = max(minimum_type, (int)((PyArrayObject *)ip)->descr->type_num);
+            Py_DECREF(ip);
+            return result;
 	} else {
 	    if (PySequence_Length(op)<0)
 		PyErr_Clear();
@@ -2243,7 +2299,7 @@ static int array_objecttype(PyObject *op, int minimum_type, int savespaceflag) {
 	}
     }
 	
-    if (PyString_Check(op)) {
+    if (PyString_Check(op) || PyUnicode_Check(op)) {
 	return max(minimum_type, (int)PyArray_CHAR);
     }
 	
@@ -2287,7 +2343,7 @@ static int Assign_Array(PyArrayObject *self, PyObject *v) {
 	
     while(--l >= 0)
 	{
-	    e=PySequence_GetItem(v,l);
+            e=PySequence_GetItem(v,l);
 	    if (e == NULL) return -1;
 	    r = PySequence_SetItem((PyObject*)self,l,e);
 	    Py_DECREF(e);
@@ -2354,6 +2410,28 @@ PyObject *PyArray_FromScalar(PyObject *op, int type) {
     }
 }
 
+int PyArray_ValidType(int type) {
+    switch(type) {
+    case 'c': 
+    case 'b': 
+    case '1': 
+    case 's':
+    case 'w': 
+    case 'i':
+    case 'u':
+    case 'l':
+    case 'f': 
+    case 'd': 
+    case 'F': 
+    case 'D': 
+    case 'O': 
+        break;
+    default:
+        if (type >= PyArray_NTYPES) return 0;
+    }
+    return 1;
+}
+
 PyObject *PyArray_Cast(PyArrayObject *mp, int type) {
     PyArrayObject *rp, *tmp;
 	
@@ -2393,18 +2471,11 @@ PyObject *array_fromobject(PyObject *op_in, int type, int min_depth, int max_dep
     /* Code that returns the object to convert for a non multiarray input object 
        from the __array__ attribute of the object. */
     if (!PyArray_Check(op_in) && PyObject_HasAttrString(op_in, "__array__")) {
-	PyObject *ap, *arglist;
-		
 	if (real_type == PyArray_NOTYPE) {
-	    arglist = Py_BuildValue("()");
+	    op = PyObject_CallMethod(op_in, "__array__", NULL);
 	} else {
-	    arglist = Py_BuildValue("(c)", real_type);
+	    op = PyObject_CallMethod(op_in, "__array__", "c", real_type);
 	}
-	ap = PyObject_GetAttrString(op_in, "__array__");
-	op = PyEval_CallObject(ap, arglist);
-	Py_DECREF(ap);
-	Py_DECREF(arglist);
-		
 	if (op == NULL) return NULL;
     } else {
 	op = op_in;
@@ -2456,6 +2527,11 @@ PyObject *array_fromobject(PyObject *op_in, int type, int min_depth, int max_dep
     /* The input object is either not an array or it is an object array, 
        or the desired output type is object. */ 
     else {
+	if (PyUnicode_Check(op)) {
+	    PyErr_SetString(PyExc_ValueError, "Unicode objects not supported.");
+	    Py_DECREF(op);
+	    return NULL;
+	}
 	r = Array_FromSequence(op, real_type, min_depth,max_depth);
 	if (r == NULL && min_depth <= 0) {
 	    PyErr_Clear();
@@ -2512,11 +2588,14 @@ extern int PyArray_CanCastSafely(int fromtype, int totype) {
 	return (totype >= PyArray_SHORT);
     case PyArray_SBYTE:
     case PyArray_SHORT:
+    case PyArray_USHORT:
 	return (totype > fromtype);
     case PyArray_INT:
     case PyArray_LONG:
 	/*Shouldn't allow Longs to be cast to Ints, not safe on 64-bit machines!*/
 	return (totype >= PyArray_INT) && (totype != PyArray_FLOAT); 
+    case PyArray_UINT:
+        return (totype >= PyArray_FLOAT); 
     case PyArray_FLOAT:
 	return (totype > PyArray_FLOAT);
     case PyArray_DOUBLE:
@@ -2692,7 +2771,7 @@ extern PyObject *PyArray_Take(PyObject *self0, PyObject *indices0, int axis) {
 		PyErr_SetString(PyExc_IndexError, "Index out of range for array");
 		goto fail;
 	    }
-	    memcpy(dest, src+tmp*chunk, chunk);
+	    memmove(dest, src+tmp*chunk, chunk);
 	    dest += chunk;
 	}
 	src += chunk*max_item;
@@ -2743,16 +2822,17 @@ extern PyObject *PyArray_Put(PyObject *self0, PyObject *indices0,
 							   self->descr->type, 0, 0);
     if (values == NULL) goto fail;
     nv = PyArray_SIZE(values);
-
-    for(i=0; i<ni; i++) {
-	src = values->data + chunk * (i % nv);
-	tmp = ((long *)(indices->data))[i];
-	if (tmp < 0) tmp = tmp+max_item;
-	if ((tmp < 0) || (tmp >= max_item)) {
-	    PyErr_SetString(PyExc_IndexError, "Index out of range for array");
-	    goto fail;
-	}
-	memcpy(dest + tmp * chunk, src, chunk);
+    if (nv > 0) { /* nv == 0 for a null array */
+        for(i=0; i<ni; i++) {
+            src = values->data + chunk * (i % nv);
+            tmp = ((long *)(indices->data))[i];
+            if (tmp < 0) tmp = tmp+max_item;
+            if ((tmp < 0) || (tmp >= max_item)) {
+                PyErr_SetString(PyExc_IndexError, "Index out of range for array");
+                goto fail;
+            }
+            memmove(dest + tmp * chunk, src, chunk);
+        }
     }
 
     Py_XDECREF(values);
@@ -2799,14 +2879,15 @@ extern PyObject *PyArray_PutMask(PyObject *self0, PyObject *mask0,
     values = (PyArrayObject *)PyArray_ContiguousFromObject(values0, 
 							   self->descr->type, 0, 0);
     if (values == NULL) goto fail;
-    nv = PyArray_SIZE(values);
-
-    for(i=0; i<ni; i++) {
-	src = values->data + chunk * (i % nv);
-	tmp = ((long *)(mask->data))[i];
-	if (tmp) {
-	    memcpy(dest + i * chunk, src, chunk);
-	}
+    nv = PyArray_SIZE(values);   /* zero if null array */
+    if (nv > 0) {
+        for(i=0; i<ni; i++) {
+            src = values->data + chunk * (i % nv);
+            tmp = ((long *)(mask->data))[i];
+            if (tmp) {
+                memmove(dest + i * chunk, src, chunk);
+            }
+        }
     }
 
     Py_XDECREF(values);
