@@ -114,7 +114,7 @@ extern PyObject *PyArray_Concatenate(PyObject *op) {
 	
     data = ret->data;
     for(i=0; i<n; i++) {
-	memcpy(data, mps[i]->data, PyArray_NBYTES(mps[i]));
+	memmove(data, mps[i]->data, PyArray_NBYTES(mps[i]));
 	data += PyArray_NBYTES(mps[i]);
     }
 	
@@ -152,7 +152,7 @@ extern PyObject *PyArray_Transpose(PyArrayObject *ap, PyObject *op) {
     long *axes, axis;
     int i, n;
     int *permutation = NULL;
-    PyArrayObject *ret;
+    PyArrayObject *ret = NULL;
 	
     if (op == Py_None) {
       n = ap->nd;
@@ -262,7 +262,7 @@ extern PyObject *PyArray_Repeat(PyObject *aop, PyObject *op, int axis) {
     for(i=0; i<n_outer; i++) {
 	for(j=0; j<n; j++) {
 	    for(k=0; k<counts[j]; k++) {
-		memcpy(new_data, old_data, chunk);
+		memmove(new_data, old_data, chunk);
 		new_data += chunk;
 	    }
 	    old_data += chunk;
@@ -366,7 +366,7 @@ extern PyObject *PyArray_Choose(PyObject *ip, PyObject *op) {
 	}
 	offset = i*elsize;
 	if (offset >= sizes[mi]) {offset = offset % sizes[mi]; }
-	memcpy(ret_data, mps[mi]->data+offset, elsize);
+	memmove(ret_data, mps[mi]->data+offset, elsize);
 	ret_data += elsize; self_data++;
     }
 	
@@ -391,14 +391,16 @@ extern PyObject *PyArray_Choose(PyObject *ip, PyObject *op) {
 int fname(type *ip1, type *ip2) { return *ip1 < *ip2 ? -1 : *ip1 == *ip2 ? 0 : 1; }
 
 COMPARE(DOUBLE_compare, double)
-     COMPARE(FLOAT_compare, float)
-     COMPARE(SHORT_compare, short)
-     COMPARE(INT_compare, int)
-     COMPARE(LONG_compare, long)
-     COMPARE(BYTE_compare, signed char)
-     COMPARE(UNSIGNEDBYTE_compare, unsigned char)
+COMPARE(FLOAT_compare, float)
+COMPARE(SHORT_compare, short)
+COMPARE(UNSIGNEDSHORT_compare, unsigned short)
+COMPARE(INT_compare, int)
+COMPARE(UINT_compare, int)
+COMPARE(LONG_compare, long)
+COMPARE(BYTE_compare, signed char)
+COMPARE(UNSIGNEDBYTE_compare, unsigned char)
 
-     int OBJECT_compare(PyObject **ip1, PyObject **ip2) { 
+int OBJECT_compare(PyObject **ip1, PyObject **ip2) { 
     return PyObject_Compare(*ip1, *ip2);
 }
 
@@ -410,7 +412,9 @@ static CompareFunction compare_functions[] =
     (CompareFunction)UNSIGNEDBYTE_compare,
     (CompareFunction)BYTE_compare,
     (CompareFunction)SHORT_compare,
+    (CompareFunction)UNSIGNEDSHORT_compare,
     (CompareFunction)INT_compare,
+    (CompareFunction)UINT_compare,
     (CompareFunction)LONG_compare,
     (CompareFunction)FLOAT_compare,
     (CompareFunction)DOUBLE_compare,
@@ -589,10 +593,12 @@ extern PyObject *PyArray_BinarySearch(PyObject *op1, PyObject *op2) {
 }
 
 DOT_PRODUCT(SHORT_DotProduct, short)
-     DOT_PRODUCT(INT_DotProduct, int)
-     DOT_PRODUCT(LONG_DotProduct, long)
-     DOT_PRODUCT(FLOAT_DotProduct, float)
-     DOT_PRODUCT(DOUBLE_DotProduct, double)
+DOT_PRODUCT(UNSIGNEDSHORT_DotProduct, unsigned short)
+DOT_PRODUCT(INT_DotProduct, int)
+DOT_PRODUCT(UINT_DotProduct, int)
+DOT_PRODUCT(LONG_DotProduct, long)
+DOT_PRODUCT(FLOAT_DotProduct, float)
+DOT_PRODUCT(DOUBLE_DotProduct, double)
 
 #define CDOT_PRODUCT(name, number) \
 	static void name(char *ip1, int is1, char *ip2, int is2, char *op, int n) { \
@@ -630,9 +636,21 @@ DOT_PRODUCT(SHORT_DotProduct, short)
 
 typedef void (DotFunction) (char *, int, char *, int, char *, int);
 
-static DotFunction *matrixMultiplyFunctions[] = {NULL,NULL,NULL,SHORT_DotProduct,INT_DotProduct,LONG_DotProduct,
-						 FLOAT_DotProduct,DOUBLE_DotProduct, 
-						 CFLOAT_DotProduct, CDOUBLE_DotProduct, OBJECT_DotProduct};
+static DotFunction *matrixMultiplyFunctions[] = {
+    NULL,
+    NULL,
+    NULL,
+    SHORT_DotProduct,
+    UNSIGNEDSHORT_DotProduct,
+    INT_DotProduct,
+    UINT_DotProduct,
+    LONG_DotProduct,
+    FLOAT_DotProduct,
+    DOUBLE_DotProduct, 
+    CFLOAT_DotProduct, 
+    CDOUBLE_DotProduct, 
+    OBJECT_DotProduct
+};
 
 extern PyObject *PyArray_InnerProduct(PyObject *op1, PyObject *op2) {
     PyArrayObject *ap1, *ap2, *ret;
@@ -956,29 +974,38 @@ typedef int (*ArgFunction) (void*, long, long*);
 for(i=1; i<n; i++) { if (ip[i] op mp) { mp = ip[i]; *ap=i; } } return 0;}
 
 ARGFUNC(DOUBLE_argmax, double, >)
-     ARGFUNC(FLOAT_argmax, float, >)
-     ARGFUNC(LONG_argmax, long, >)
-     ARGFUNC(INT_argmax, int, >)
-     ARGFUNC(SHORT_argmax, short, >)
-     ARGFUNC(BYTE_argmax, signed char, >) 
-     ARGFUNC(UNSIGNEDBYTE_argmax, unsigned char, >) 
-     int OBJECT_argmax(PyObject **ip, long n, long *ap) {
+ARGFUNC(FLOAT_argmax, float, >)
+ARGFUNC(LONG_argmax, long, >)
+ARGFUNC(INT_argmax, int, >)
+ARGFUNC(UINT_argmax, int, >)
+ARGFUNC(SHORT_argmax, short, >)
+ARGFUNC(UNSIGNEDSHORT_argmax, unsigned short, >)
+ARGFUNC(BYTE_argmax, signed char, >) 
+ARGFUNC(UNSIGNEDBYTE_argmax, unsigned char, >) 
+
+int OBJECT_argmax(PyObject **ip, long n, long *ap) {
     long i; PyObject *mp=ip[0]; *ap=0;
     for(i=1; i<n; i++) { 
-	if (PyObject_Compare(ip[i],mp) > 0) { mp = ip[i]; *ap=i; } 
+	    if (PyObject_Compare(ip[i],mp) > 0) { mp = ip[i]; *ap=i; } 
     } 
     return 0;
 }
 
-static ArgFunction argmax_functions[] = {NULL,
-                                (ArgFunction) UNSIGNEDBYTE_argmax,
-                                (ArgFunction) BYTE_argmax,
-                                (ArgFunction)SHORT_argmax,
-			        (ArgFunction)INT_argmax,(ArgFunction)LONG_argmax,
-				(ArgFunction)FLOAT_argmax,(ArgFunction)DOUBLE_argmax,
-				NULL,
-				NULL,
-				(ArgFunction)OBJECT_argmax};
+static ArgFunction argmax_functions[] = {
+    NULL,
+    (ArgFunction) UNSIGNEDBYTE_argmax,
+    (ArgFunction) BYTE_argmax,
+    (ArgFunction)SHORT_argmax,
+    (ArgFunction)UNSIGNEDSHORT_argmax,
+    (ArgFunction)INT_argmax,
+    (ArgFunction)UINT_argmax,
+    (ArgFunction)LONG_argmax,
+    (ArgFunction)FLOAT_argmax,
+    (ArgFunction)DOUBLE_argmax,
+    NULL,
+    NULL,
+    (ArgFunction)OBJECT_argmax
+};
 
 extern PyObject *PyArray_ArgMax(PyObject *op) {
     PyArrayObject *ap, *rp;
@@ -1024,7 +1051,9 @@ extern PyObject *PyArray_ArgMax(PyObject *op) {
 	      ARGFUNC(FLOAT_argmin, float, <)
 	      ARGFUNC(LONG_argmin, long, <)
 	      ARGFUNC(INT_argmin, int, <)
+	      ARGFUNC(UINT_argmin, int, <)
 	      ARGFUNC(SHORT_argmin, short, <)
+              ARGFUNC(UNSIGNEDSHORT_argmin, unsigned short, <)
 
 	      int OBJECT_argmin(PyObject **ip, long n, PyObject **mp, long *ap) {
 	      long i; *mp=ip[0]; *ap=0;
@@ -1054,12 +1083,18 @@ array_array(PyObject *ignored, PyObject *args, PyObject *kws)
 	type = PyArray_NOTYPE;
     } else {
 	tp = PyString_AsString(tpo);
-	if (tp == NULL) {
-	    PyErr_SetString(PyExc_TypeError, "typecode argument must be a string.");
+	if ((tp == NULL) || (PyString_Size(tpo) > 1) ) {
+	    PyErr_SetString(PyExc_TypeError, "typecode argument must be a valid type.");
 	    return NULL;
 	}
 	if (tp[0] == 0) type = PyArray_NOTYPE;
-	else type = tp[0];
+	else {
+            type = tp[0];
+            if (!PyArray_ValidType(type)) {
+                PyErr_SetString(PyExc_TypeError, "typecode argument must be a valid type.");
+                return NULL;
+            }
+        }
     }
 
     if (savespace != 0)
@@ -1079,9 +1114,9 @@ array_array(PyObject *ignored, PyObject *args, PyObject *kws)
     return ret;
 }
 
-static char doc_zeros[] = "zeros((d1,...,dn),typecode,savespace) will return a new array of shape (d1,...,dn) and type typecode (default 'l') with all it's entries initialized to zero.  If savespace (default 0) is nonzero the array will be a spacesaver array.";
+static char doc_zeros[] = "zeros((d1,...,dn),typecode='l',savespace=0) will return a new array of shape (d1,...,dn) and type typecode with all it's entries initialized to zero.  If savespace is nonzero the array will be a spacesaver array.";
 
-static PyObject *array_zeros(PyObject *ignored, PyObject *args) {
+static PyObject *array_zeros(PyObject *ignored, PyObject *args, PyObject *kwds) {
     PyObject *op, *sequence, *tpo=Py_None;
     PyArrayObject *ret;
     char type_char='l';
@@ -1089,8 +1124,10 @@ static PyObject *array_zeros(PyObject *ignored, PyObject *args) {
     int i, nd, n, dimensions[MAX_DIMS];
     int savespace=0;
     char all_zero[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
+    static char *kwlist[] = {"shape", "typecode", "savespace", NULL};
 	
-    if (!PyArg_ParseTuple(args, "O|Oi", &sequence, &tpo, &savespace)) {
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|Oi", kwlist,
+                                     &sequence, &tpo, &savespace)) {
 	return NULL;
     }
     if (tpo != Py_None) {
@@ -1130,7 +1167,7 @@ static PyObject *array_zeros(PyObject *ignored, PyObject *args) {
 	dptr = ret->data;
 	n = PyArray_SIZE(ret);
 	for(i=0; i<n; i++) {
-	    memcpy(dptr, ret->descr->zero, ret->descr->elsize);
+	    memmove(dptr, ret->descr->zero, ret->descr->elsize);
 	    dptr += ret->descr->elsize;
 	}
     }
@@ -1179,19 +1216,21 @@ static PyObject *array_fromString(PyObject *ignored, PyObject *args, PyObject *k
     if ((ret = (PyArrayObject *)PyArray_FromDims(1, &n, *type)) == NULL) 
 	return NULL;
 	
-    memcpy(ret->data, data, n*ret->descr->elsize);
+    memmove(ret->data, data, n*ret->descr->elsize);
     PyArray_INCREF(ret);
     return (PyObject *)ret;
 }
 
 static char doc_take[] = "take(a, indices, axis=0).  Selects the elements in indices from array a along the given axis.";
 
-static PyObject *array_take(PyObject *dummy, PyObject *args) {
+static PyObject *array_take(PyObject *dummy, PyObject *args, PyObject *kwds) {
     int dimension;
     PyObject *a, *indices, *ret;
+    static char *kwlist[] = {"array", "indices", "axis", NULL};
 	
     dimension=0;
-    if (!PyArg_ParseTuple(args, "OO|i", &a, &indices, &dimension))
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO|i", kwlist, 
+                                     &a, &indices, &dimension))
 	return NULL;
 	
     ret = PyArray_Take(a, indices, dimension);
@@ -1249,11 +1288,13 @@ static PyObject *array_concatenate(PyObject *dummy, PyObject *args) {
 
 static char doc_transpose[] = "transpose(a, axes=None)";
 
-static PyObject *array_transpose(PyObject *dummy, PyObject *args) {
+static PyObject *array_transpose(PyObject *dummy, PyObject *args, PyObject *kwds) {
     PyObject *shape=Py_None, *ret, *a0;
     PyArrayObject *a;
+    static char *kwlist[] = {"array", "axes", NULL};
 	
-    if (!PyArg_ParseTuple(args, "O|O", &a0, &shape)) return NULL;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O", kwlist, 
+                                     &a0, &shape)) return NULL;
     if ((a = (PyArrayObject *)PyArray_FromObject(a0,
 						 PyArray_NOTYPE,0,0))
 	==NULL) return NULL;
@@ -1266,11 +1307,13 @@ static PyObject *array_transpose(PyObject *dummy, PyObject *args) {
 
 static char doc_repeat[] = "repeat(a, n, axis=0)";
 
-static PyObject *array_repeat(PyObject *dummy, PyObject *args) {
+static PyObject *array_repeat(PyObject *dummy, PyObject *args, PyObject *kwds) {
     PyObject *shape, *a0;
     int axis=0;
+    static char *kwlist[] = {"array", "shape", "axis", NULL};
 	
-    if (!PyArg_ParseTuple(args, "OO|i", &a0, &shape, &axis)) return NULL;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO|i", kwlist, 
+                                     &a0, &shape, &axis)) return NULL;
 	
     return PyArray_Repeat(a0, shape, axis);
 }
@@ -1345,16 +1388,19 @@ static PyObject *array_fastCopyAndTranspose(PyObject *dummy, PyObject *args) {
     return PyArray_fastCopyAndTranspose(a0);
 }
 
-static char doc_correlate[] = "cross_correlate(a,v)";
+static char doc_correlate[] = "cross_correlate(a,v, mode=0)";
 
-static PyObject *array_correlate(PyObject *dummy, PyObject *args) {
+static PyObject *array_correlate(PyObject *dummy, PyObject *args, PyObject *kwds) {
     PyObject *shape, *a0;
     int mode=0;
+    static char *kwlist[] = {"a", "v", "mode", NULL};
 	
-    if (!PyArg_ParseTuple(args, "OO|i", &a0, &shape, &mode)) return NULL;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO|i", kwlist, 
+                                     &a0, &shape, &mode)) return NULL;
 	
     return PyArray_Correlate(a0, shape, mode);
 }
+
 static char doc_argmax[] = "argmax(a)";
 
 static PyObject *array_argmax(PyObject *dummy, PyObject *args) {
@@ -1439,10 +1485,9 @@ array_arange(PyObject *ignored, PyObject *args, PyObject *kws) {
     rptr = ((PyArrayObject *)range)->data;
     elsize = ((PyArrayObject *)range)->descr->elsize;
     type = ((PyArrayObject *)range)->descr->type_num;
-    value = start;
     for (i=0; i < length; i++) {
-	dbl_descr->cast[type](&value, 0, rptr, 0, 1);
-	value += step;
+        value = start + i*step;
+	dbl_descr->cast[type]((char*)&value, 0, rptr, 0, 1);
 	rptr += elsize;
     }
     
@@ -1461,43 +1506,46 @@ array_arange(PyObject *ignored, PyObject *args, PyObject *kws) {
       }
 *****/
 
-static char doc_set_string_function[] = "set_string_function(f) sets the python function f to be the function used to obtain a pretty printable string version of a array whenever a array is printed.  f(M) should expect a array argument M, and should return a string consisting of the desired representation of M for printing.";
+static char doc_set_string_function[] = "set_string_function(f, repr=1) sets the python function f to be the function used to obtain a pretty printable string version of a array whenever a array is printed.  f(M) should expect a array argument M, and should return a string consisting of the desired representation of M for printing.";
 
-static PyObject *array_set_string_function(PyObject *dummy, PyObject *args) {
+static PyObject *array_set_string_function(PyObject *dummy, PyObject *args, PyObject *kwds) {
     PyObject *op;
     int repr=1;
-    if(!PyArg_ParseTuple(args, "O|i", &op, &repr)) return NULL; 
+    static char *kwlist[] = {"f", "repr", NULL};
+
+    if(!PyArg_ParseTupleAndKeywords(args, kwds, "O|i", kwlist, 
+                                    &op, &repr)) return NULL; 
     PyArray_SetStringFunction(op, repr);
     Py_INCREF(Py_None);
     return Py_None;
 }
 
 static struct PyMethodDef array_module_methods[] = {
-    {"set_string_function", array_set_string_function, 1, doc_set_string_function},
+    {"set_string_function", array_set_string_function, METH_VARARGS|METH_KEYWORDS, doc_set_string_function},
 		
-    {"array",	(PyCFunction)array_array, 3, doc_array},
-    {"arange",  (PyCFunction)array_arange, 3, doc_arange},
-    {"zeros",	array_zeros, 1, doc_zeros},
-    {"fromstring",	array_fromString, 3, doc_fromString},
-    {"take",	array_take, 1, doc_take},
-    {"put",		array_put, 1, doc_put},
-    {"putmask",	array_putmask, 1, doc_putmask},
-    {"reshape",	array_reshape, 1, doc_reshape},
-    {"concatenate",	array_concatenate, 1, doc_concatenate},
-    {"transpose",	array_transpose, 1, doc_transpose},
-    {"repeat",	array_repeat, 1, doc_repeat},
-    {"choose",	array_choose, 1, doc_choose},
+    {"array",	(PyCFunction)array_array, METH_VARARGS|METH_KEYWORDS, doc_array},
+    {"arange",  (PyCFunction)array_arange, METH_VARARGS|METH_KEYWORDS, doc_arange},
+    {"zeros",	(PyCFunction)array_zeros, METH_VARARGS|METH_KEYWORDS, doc_zeros},
+    {"fromstring",(PyCFunction)array_fromString, METH_VARARGS|METH_KEYWORDS, doc_fromString},
+    {"take",	(PyCFunction)array_take, METH_VARARGS|METH_KEYWORDS, doc_take},
+    {"put",	(PyCFunction)array_put, METH_VARARGS, doc_put},
+    {"putmask",	(PyCFunction)array_putmask, METH_VARARGS, doc_putmask},
+    {"reshape",	(PyCFunction)array_reshape, METH_VARARGS, doc_reshape},
+    {"concatenate",	(PyCFunction)array_concatenate, METH_VARARGS, doc_concatenate},
+    {"transpose",	(PyCFunction)array_transpose, METH_VARARGS|METH_KEYWORDS, doc_transpose},
+    {"repeat",	(PyCFunction)array_repeat, METH_VARARGS|METH_KEYWORDS, doc_repeat},
+    {"choose",	(PyCFunction)array_choose, METH_VARARGS, doc_choose},
 	
-    {"sort",	array_sort, 1, doc_sort},
-    {"argsort",	array_argsort, 1, doc_argsort},
-    {"binarysearch",	array_binarysearch, 1, doc_binarysearch},
+    {"sort",	(PyCFunction)array_sort, METH_VARARGS, doc_sort},
+    {"argsort",	(PyCFunction)array_argsort, METH_VARARGS, doc_argsort},
+    {"binarysearch",	(PyCFunction)array_binarysearch, METH_VARARGS, doc_binarysearch},
 	
-    {"argmax",	array_argmax, 1, doc_argmax},
-    {"innerproduct",	array_innerproduct, 1, doc_innerproduct}, 
-    {"matrixproduct",	array_matrixproduct, 1, doc_matrixproduct}, 
-    {"_fastCopyAndTranspose", array_fastCopyAndTranspose, 1, doc_fastCopyAndTranspose},
-    {"cross_correlate", array_correlate, 1, doc_correlate},
-    /*  {"arrayMap",	array_arrayMap, 1, doc_arrayMap},*/
+    {"argmax",	(PyCFunction)array_argmax, METH_VARARGS, doc_argmax},
+    {"innerproduct",	(PyCFunction)array_innerproduct, METH_VARARGS, doc_innerproduct}, 
+    {"matrixproduct",	(PyCFunction)array_matrixproduct, METH_VARARGS, doc_matrixproduct}, 
+    {"_fastCopyAndTranspose", (PyCFunction)array_fastCopyAndTranspose, METH_VARARGS, doc_fastCopyAndTranspose},
+    {"cross_correlate", (PyCFunction)array_correlate, METH_VARARGS | METH_KEYWORDS, doc_correlate},
+    /*  {"arrayMap",	(PyCFunction)array_arrayMap, 1, doc_arrayMap},*/
 	
     {NULL,		NULL, 0}		/* sentinel */
 };
