@@ -37,6 +37,16 @@ class BasicNumericTestCase (unittest.TestCase):
         assert w.itemsize() == 4
         assert w.iscontiguous()
         assert w.astype(Numeric.Float).typecode() == Numeric.Float
+        assert Numeric.shape(alist) == Numeric.shape(x)
+        assert Numeric.rank(x) == 1
+        assert Numeric.rank(1) == 0
+        assert Numeric.size(x) == len(alist)
+        x2 = Numeric.array([x,x])
+        assert Numeric.rank(x2) == 2
+        assert Numeric.size(x2, 0) == 2
+        assert Numeric.size(x2, 1) == len(x)
+        assert Numeric.size(x2) == 2 * len(x)
+        assert eq(Numeric.average(alist), 2.0)
 
     def testTypecodeSpecifying(self):
         'Test construction using the type codes.'
@@ -92,6 +102,15 @@ class BasicNumericTestCase (unittest.TestCase):
         assert eq(b[2:10], [[1,2,3,4],[5,6,7,8]])
         assert eq(b[2:10, ...], [[1,2,3,4],[5,6,7,8]])
 
+    def testBackwardStrides (self):
+        "Test backward stride operations like x[3::-1]"
+        m = Numeric.array([1.,2.,3.,4.])
+        assert eq(m[2:1:-1], [3.])
+        assert eq(m[2::-1], [3.,2.,1.])
+        assert eq(m[1::-1], [2.,1.])
+        assert eq(m[4::-1], m[3::-1])
+        assert eq(m[4::-2], [4.,2.])
+
     def testOperators (self):
         "Test the operators +, -, *, /, %, ^, &, |"
         x = Numeric.array([1.,2.,3.,4.,5.,6.])
@@ -195,6 +214,39 @@ class BasicNumericTestCase (unittest.TestCase):
         assert Numeric.multiply.reduce([]) == 1
         assert Numeric.minimum.reduce(a) == 0
         assert Numeric.maximum.reduce(a) == 5
+
+    def testAverage (self):
+        "Test of average function."
+        a = Numeric.arange(6)
+        assert eq(Numeric.average(a), 15.0/6.0)
+        assert eq(Numeric.average(a, weights=[1,1,1,2,1,0]), 
+                  (0*1+1*1+2*1+3*2+4*1+5*0)/(1+1+1+2+1+0.0))
+        assert eq(Numeric.average(a, weights=[1,1,1,2,1,0],returned=1)[0], 
+                  (0*1+1*1+2*1+3*2+4*1+5*0)/(1+1+1+2+1+0.0))
+        assert eq(Numeric.average(a, weights=[1,1,1,2,1,0],returned=1)[1], 
+                  1+1+1+2+1+0)
+        b = Numeric.arange(6) * 3
+        assert eq(Numeric.average([a,b]), (a + b) / 2.)    
+        assert eq(Numeric.average([a,b],1), [15/6.0, 45/6.0])   
+        assert eq(Numeric.average([a,b], axis=0, weights=[3,1]), (3.0*a+b)/4.)
+        w = Numeric.ones(Numeric.shape([a,b]), Numeric.Float)/2.0
+        assert eq(Numeric.average([a,b], axis=1), 
+                  Numeric.average([a,b], weights=w, axis=1)
+                  )
+        c = a[:]
+        c.shape=(3,2)
+        assert eq(Numeric.average(a), Numeric.average(c, axis=None))
+        assert eq(Numeric.average(a[::2]), (0+2+4)/3.0)
+        r1, w1 = Numeric.average([[a,b],[b,a]], axis=1, returned=1)
+        assert Numeric.shape(r1) == Numeric.shape(w1)
+        assert r1.shape == w1.shape
+        r2, w2 = Numeric.average(Numeric.ones((2,2,3)), axis=0, weights=[3,1], returned=1)
+        assert Numeric.shape(w2) == Numeric.shape(r2)
+        r2, w2 = Numeric.average(Numeric.ones((2,2,3)), returned=1)
+        assert Numeric.shape(w2) == Numeric.shape(r2)
+        r2, w2 = Numeric.average(Numeric.ones((2,2,3)), weights=Numeric.ones((2,2,3)), returned=1)
+        assert Numeric.shape(w2) == Numeric.shape(r2)
+        
         
     def testSpacesaver (self):
         "Test the spacesaver property (Travis Oliphant)"
@@ -387,6 +439,10 @@ class ArrayFunctionsTestCase (unittest.TestCase):
         c[2] = a[2]*b
         c[3] = a[3]*b
         assert eq(c, Numeric.outerproduct(a,b))
+        a = Numeric.arange(8)
+        b = a[::2]
+        c = Numeric.outerproduct(b,b)
+        assert eq(c, Numeric.outerproduct([0,2,4,6],[0,2,4,6]))
 
     def testChoose (self):
         "Test the choose function."
@@ -503,19 +559,11 @@ class ArrayRelationsToOtherModulesTestCase (unittest.TestCase):
         x[1] = 4
         assert not Numeric.allclose(x,c1)
         c2 = copy.copy(y)
-        assert id(c2[1]) == id(x)
+        assert id(c2[1]) == id(Texx)
         c3 = copy.deepcopy(y)
         assert id(c3[1]) != id(x)
         assert Numeric.allclose(c3[1], x)
         
-def suite():
-    testsuite = unittest.makeSuite(BasicNumericTestCase, 'test')
-    testsuite.addTest(unittest.makeSuite(ArrayCreatorsTestCase, 'test'))
-    testsuite.addTest(unittest.makeSuite(ArrayFunctionsTestCase, 'test'))
-    testsuite.addTest(unittest.makeSuite(UniversalFunctionsTestCase, 'test'))
-    testsuite.addTest(unittest.makeSuite(ArrayRelationsToOtherModulesTestCase, 'test'))
-    return testsuite
 
 if __name__ == '__main__':
-    r = unittest.TextTestRunner()
-    r.run(suite())
+    unittest.main()
\ No newline at end of file
